// This is the main DLL file.

#include "stdafx.h"

#include "ConsoleLib.h"


#include <stdlib.h>
#include <Windows.h>
#include <stdio.h>



HANDLE wHnd;
HANDLE rHnd;

CHAR_INFO* consoleBuffer;
COORD conSize;
COORD conWritePos = {0, 0};
SMALL_RECT conWriteArea;

bool needsUpdate = true;

extern "C" __declspec(dllexport) void InitConsole(char* title, int width, int height)
{
	//get the handles
	wHnd = GetStdHandle(STD_OUTPUT_HANDLE);
	rHnd = GetStdHandle(STD_INPUT_HANDLE);

	//set the window title
	SetConsoleTitle(title);
	
	//initialize the buffer
	consoleBuffer = new CHAR_INFO[width*height];
	
	//set the window size
	conWriteArea.Top = 0;
	conWriteArea.Left = 0;
	conWriteArea.Right = width - 1;
	conWriteArea.Bottom = height - 1;
	SetConsoleWindowInfo(wHnd, TRUE, &conWriteArea);
	
	//set buffer size
	conSize.X = width;
	conSize.Y = height;
	SetConsoleScreenBufferSize(wHnd, conSize);

	//move cursor to bottom right
	for(int i = 0; i < conSize.Y-1; i++)
		printf("\n");
	for(int i = 0; i < conSize.X-1; i++)
		printf(" ");
}

extern "C" __declspec(dllexport) void SetCC(int x, int y, unsigned char ch, unsigned char col)
{
	needsUpdate = true;
	consoleBuffer[x+y*conSize.X].Char.AsciiChar = ch;
	consoleBuffer[x+y*conSize.X].Attributes = col;
}

extern "C" __declspec(dllexport) void SetChar(int x, int y, unsigned char c)
{
	needsUpdate=true;
	consoleBuffer[x+y*conSize.X].Char.AsciiChar = c;
}

extern "C" __declspec(dllexport) unsigned char GetChar(int x, int y)
{
	return consoleBuffer[x+y*conSize.X].Char.AsciiChar;
}

extern "C" __declspec(dllexport) void SetColor(int x, int y, unsigned char color)
{
	needsUpdate=true;
	consoleBuffer[x+y*conSize.X].Attributes = color;
}

extern "C" __declspec(dllexport) unsigned char GetColor(int x, int y)
{
	return (unsigned char)consoleBuffer[x+y*conSize.X].Attributes;
}

extern "C" __declspec(dllexport) void UpdateConsole()
{
	if(needsUpdate)
	{
		WriteConsoleOutput(wHnd, consoleBuffer, conSize, conWritePos, &conWriteArea);
		needsUpdate=false;
	}
}

extern "C" __declspec(dllexport) void ForceUpdate()
{
	WriteConsoleOutput(wHnd, consoleBuffer, conSize, conWritePos, &conWriteArea);
}

extern "C" __declspec(dllexport) void WriteGarbage()
{
	needsUpdate=true;
	for(int y = 0; y < 16; y++)
	{
		for(int x = 0; x < 16; x++)
		{
			SetChar(x, y, x+y*16);
			SetColor(x, y, FOREGROUND_RED|FOREGROUND_GREEN|FOREGROUND_BLUE);
		}
	}
	for(int x = 0; x < 16; x++)
	{
		SetCC(x, 18, 219, (unsigned char)x);
		SetCC(x, 16, 48+((x/10)%10), FOREGROUND_RED|FOREGROUND_GREEN|FOREGROUND_BLUE);
		SetCC(x, 17, 48+(x%10), FOREGROUND_RED|FOREGROUND_GREEN|FOREGROUND_BLUE);
	}
}

extern "C" __declspec(dllexport) void QuitConsole()
{
	delete [] consoleBuffer;
}

//extern "C" __declspec(dllexport) void GetEvents(